/**
 * @file toggle.c
 * This program demonstrates starting COG code with C in it
 * from C. The cog makes all IO except 30/31 toggle.
 *
 * Copyright (c) 2011 Parallax, Inc.
 * MIT Licensed (see at end of file for exact terms)
 */

#include <stdio.h>
#include <unistd.h>
#include <propeller.h>
#include "toggle.h"

#ifdef FDX
extern _Driver _FullDuplexSerialDriver;

/* This is a list of all drivers we can use in the
 * program. The default _InitIO function opens stdin,
 * stdout, and stderr based on the first driver in
 * the list (the serial driver, for us)
 */
_Driver *_driverlist[] = {
  &_FullDuplexSerialDriver,
  NULL
};
#endif
 
/*
 * This is the structure which we'll pass to the C cog.
 * It contains a small stack for working area, and the
 * mailbox which we use to communicate. See toggle.h
 * for the definition/
 */
volatile HUBDATA Par_t par;

/*
 * function to start up a new cog running the toggle
 * code (which we've placed in the .coguser1 section)
 */
int start(volatile void *parptr)
{
    /* macros like extern_driver are in toggle.h */
    extern_driver(toggle_fw_cog);
    return loadcog(cog_driver(toggle_fw_cog), cog_size(toggle_fw_cog), parptr);
}

/*
 * main code
 * This is the code running in the LMM cog (cog 0).
 * It launches another cog to actually run the 
 * toggling code
 */
#define MIN_GAP 400000

int main (int argc,  char* argv[])
{
    int myled = 1 << 23;
    int cog;
    waitcnt(CLKFREQ+CNT);
    printf("hello, world!\n");

    /* set up the parameters for the C cog */
    /* start by waiting for 1 second */
    par.m.wait_time = CLKFREQ;
    
    /* start the new cog */
    cog = start(&par.m);
    printf("toggle cog started %d\n", cog);

    /* every 2 seconds update the flashing frequency so the
       light blinks faster and faster */
    DIRA |= myled;
    while(1) {
        waitcnt(CLKFREQ*1+CNT);
        OUTA ^= myled;
        par.m.wait_time =  par.m.wait_time >> 1;
        if (par.m.wait_time < MIN_GAP)
            par.m.wait_time = _clkfreq;
        printf("toggle count = %d\n", par.m.togglecount);
    }
    return 0;
}

/* +--------------------------------------------------------------------
 * | TERMS OF USE: MIT License
 * +--------------------------------------------------------------------
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files
 * (the "Software"), to deal in the Software without restriction,
 * including without limitation the rights to use, copy, modify, merge,
 * publish, distribute, sublicense, and/or sell copies of the Software,
 * and to permit persons to whom the Software is furnished to do so,
 * subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 * +--------------------------------------------------------------------
 */
